#!/usr/bin/python

# Copyright 2014 Max-Planck-Institut Informatik
#
#    This file is part of RINerator
# 
#    RINerator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    RINerator is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RINerator.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import st_selection
import nc_interactions
import nci_residues
import nci_graph
import sel_utils

### set program paths
reduce_cmd = 'REDUCE_INSTALLATION_DIRECTORY/reduce.3.23.130521.linuxi386'    # reduce command
probe_cmd = 'PROBE_INSTALLATION_DIRECTORY/probe.2.16.130520.linuxi386'       # probe command

if len(sys.argv) != 4 and len(sys.argv) != 5:
    print(
    '''
    USAGE: get_chains.py path_pdb path_output path_chains [path_ligands]
    
    Input: PDB file or directory with PDB files, list of chains (and ligands) to be included in the RIN(s)
    Output: PDB file(s) with added hydrogens, PROBE output file(s), RIN file(s), edge attribute files

    Parameters:
    path_pdb: PDB file or directory containing PDB files of the same protein structure
    
    path_output: directory to save generated files

    path_chains: file with chain identifiers separated by commas
    - chain identifier might be any letter or an empty string
    - examples: "A,B,I" or "" or "A,"
    
    path_ligands: [optional] file with ligand identifiers separated by new lines
    - each ligand is listed with a name, a chain identifier and a residue number separated by commas
    - examples: "NOA,I,1" or "NOA,,1"    
    '''
    )
    sys.exit()

print "Processing input..."

### set paths for files to read/write
input_path = sys.argv[1]                        # path of the original PDB file
output_path = sys.argv[2]                       # path to save everything generated by rinerator

### check if single or multiple files are given as input
single_use = None
if input_path.endswith("/") and os.access(input_path, os.R_OK):
    single_use = False
elif os.path.exists(input_path):
    single_use = True

if single_use == None:
    print "Cannot read input file or directory. Exiting..."
    sys.exit()

### read in chains
chains_file = sys.argv[3]
file_content = open(chains_file, 'r').read()
chains = [c.strip() for c in file_content.rstrip().split(",")]
if len(chains) == 0:
    print "Cannot read chains file. Exiting..."
    sys.exit()

### read in ligands
ligands = []
if len(sys.argv) == 5:
    ligand_file = sys.argv[4]
    file_content = open(ligand_file, 'r').read().rstrip().split("\n")
    for line in file_content:
	ligands.append([l.strip() for l in line.split(",")])

### create RINs
if single_use:
    ### set file names
    input_pdb_parts = input_path.split("/")
    input_path = "/".join(input_pdb_parts[:len(input_pdb_parts) - 1])
    pdb_filename = input_pdb_parts[len(input_pdb_parts) - 1]
    print "Processing single pdb file " + pdb_filename + "..."

    ### create slection list
    sel_id = pdb_filename[:-4]
    selection_lst = sel_utils.chains_to_sel_list(sel_id,chains)
    ligand_lst = sel_utils.ligands_to_sel_list(ligands)
    if len(ligand_lst) > 0:
	selection_lst += ligand_lst
    
    ### run reduce and probe
    (pdb_h_filename, probe_filename)=nc_interactions.get_reduce_probe_rsl(pdb_filename,input_path,output_path,output_path,reduce_cmd,probe_cmd)
    sif_file = pdb_h_filename[:-4] + ".sif"
    
    ### get network sif file
    (stsel_obj,nci_obj,nci_res,nci_graph)=nci_graph.pdb_to_sif(sel_id,pdb_h_filename,output_path,selection_lst,probe_filename,output_path,sif_file)
else:
    print "Processing multiple pdb files in directory " + input_path
    for pdb_filename in os.listdir(input_path):
	if pdb_filename.endswith(".pdb") or pdb_filename.endswith(".ent"):
	    ### create selection list
	    sel_id = pdb_filename[:-4]
	    selection_lst = sel_utils.chains_to_sel_list(sel_id,chains)
	    ligand_lst = sel_utils.ligands_to_sel_list(ligands)
	    if len(ligand_lst) > 0:
		selection_lst += ligand_lst
	    
	    ### check if probe and reduce were already run on this file
	    pdb_h_filename = pdb_filename[:-4]+"_h.ent"
	    probe_filename = pdb_filename[:-4]+"_h.probe"
	    if not os.path.exists(output_path+pdb_h_filename) or not os.path.exists(output_path+probe_filename):
		### run reduce and probe
		nc_interactions.get_reduce_probe_rsl(pdb_filename,input_path,output_path,output_path,reduce_cmd,probe_cmd)
	    
	    ### get network sif file
	    sif_file = pdb_h_filename[:-4] + ".sif"
	    if not os.path.exists(sif_file):
		nci_graph.pdb_to_sif(sel_id,pdb_h_filename,output_path,selection_lst,probe_filename,output_path,sif_file)
